<?php

// TODO: Maintain plugin namespace using your name or something similar
namespace LeGmask;

use Exception;
use FML\Controls\Frame;
use FML\Controls\Labels\Label_Text;
use FML\Controls\Quad;
use FML\ManiaLink;
use ManiaControl\Admin\AuthenticationManager;
use ManiaControl\Callbacks\CallbackListener;
use ManiaControl\Callbacks\Callbacks;
use ManiaControl\Callbacks\Structures\Common\StatusCallbackStructure;
use ManiaControl\Callbacks\Structures\TrackMania\OnScoresStructure;
use ManiaControl\Callbacks\Structures\TrackMania\OnWayPointEventStructure;
use ManiaControl\Callbacks\TimerListener;
use ManiaControl\Commands\CommandListener;
use ManiaControl\Communication\CommunicationAnswer;
use ManiaControl\Communication\CommunicationListener;
use ManiaControl\Manialinks\ManialinkPageAnswerListener;
use ManiaControl\Players\Player;
use ManiaControl\Players\PlayerManager;
use ManiaControl\Plugins\Plugin;
use ManiaControl\ManiaControl;
use \ManiaControl\Logger;
use ManiaControl\Settings\Setting;
use ManiaControl\Settings\SettingManager;
use ManiaControl\Utils\Formatter;
use Maniaplanet\DedicatedServer\InvalidArgumentException;



/**
 * allows you to add guest who will not earn points in cup / round mode
 *
 * @author  LeGmask
 * @version 0.2
 */
class GuestPlugin implements ManialinkPageAnswerListener, CallbackListener, CommandListener, TimerListener, CommunicationListener, Plugin {
	/*
	 * Constants
	 */

	const ID      = 146;
	const VERSION = 0.2;
	const NAME    = 'Guest Plugin';
	const AUTHOR  = 'LeGmask';
	
	// GuestPlugin Properties
	
	const SETTING_GUEST_PLUGIN_ACTIVATED   = 'GuestPlugin Activated';
	const SETTING_GUEST_PLUGIN_GUEST1      = 'Login of guest 1';
	const SETTING_GUEST_PLUGIN_GUEST2	   = 'Login of guest 2';
	const SETTING_GUEST_PLUGIN_GUEST3      = 'Login of guest 3';
	const SETTING_GUEST_PLUGIN_GUEST4      = 'Login of guest 4';
	const SETTING_GUEST_PLUGIN_GUEST5      = 'Login of guest 5';
	const SETTING_GUEST_PLUGIN_GUESTPOINT   = 'Reset guest point at the end of the round';
	
	const DEFAULT_TRACKMANIALIVE_PLUGIN = 'Drakonia\TrackmaniaLivePlugin';
		

	/**
	 * Private Properties
	 */
	public $livePlugin = false;
	/** @var ManiaControl $maniaControl */
	private $maniaControl = null;
	private $nbplayers = 0;
	private $playerstate = array();

	/**
	 * @see \ManiaControl\Plugins\Plugin::prepare()
	 */
	public static function prepare(ManiaControl $maniaControl) {
		// TODO: Implement prepare() method.
	}

	/**
	 * @see \ManiaControl\Plugins\Plugin::load()
	 */
	public function load(ManiaControl $maniaControl) {
		$this->maniaControl = $maniaControl;
		
		// Callbacks
		$this->maniaControl->getCallbackManager()->registerCallbackListener(PlayerManager::CB_PLAYERCONNECT, $this, 'handlePlayerConnect');
		$this->maniaControl->getCallbackManager()->registerCallbackListener(SettingManager::CB_SETTING_CHANGED, $this, 'updateSettings');
		$this->maniaControl->getCallbackManager()->registerCallbackListener(
            Callbacks::TM_SCORES,
            $this,
            'handleEndRoundCallback'
		);
		$this->maniaControl->getCallbackManager()->registerCallbackListener(
            PlayerManager::CB_PLAYERDISCONNECT,
            $this,
            'handlePlayerDisconnect'
        );
        $this->maniaControl->getCallbackManager()->registerCallbackListener(
            PlayerManager::CB_PLAYERINFOCHANGED,
            $this,
            'handlePlayerInfoChanged'
        );
		
		
		// Settings
		$this->maniaControl->getSettingManager()->initSetting($this, self::SETTING_GUEST_PLUGIN_ACTIVATED, true);
		$this->maniaControl->getSettingManager()->initSetting($this, self::SETTING_GUEST_PLUGIN_GUEST1, "");
		$this->maniaControl->getSettingManager()->initSetting($this, self::SETTING_GUEST_PLUGIN_GUEST2, "");
		$this->maniaControl->getSettingManager()->initSetting($this, self::SETTING_GUEST_PLUGIN_GUEST3, "");
		$this->maniaControl->getSettingManager()->initSetting($this, self::SETTING_GUEST_PLUGIN_GUEST4, "");
		$this->maniaControl->getSettingManager()->initSetting($this, self::SETTING_GUEST_PLUGIN_GUEST5, "");
		$this->maniaControl->getSettingManager()->initSetting($this, self::SETTING_GUEST_PLUGIN_GUESTPOINT, true);
		
		
		// Trackmania Live Plugin

        $this->livePlugin = $this->maniaControl->getPluginManager()->getPlugin(self::DEFAULT_TRACKMANIALIVE_PLUGIN);
		
		return true;
	}

	/**
	 * @see \ManiaControl\Plugins\Plugin::unload()
	 */
	public function unload() {
		$this->maniaControl = null;
	}
	
	/**
	 * @see \ManiaControl\Plugins\Plugin::getId()
	 */
	public static function getId() {
		return self::ID;
	}

	/**
	 * @see \ManiaControl\Plugins\Plugin::getName()
	 */
	public static function getName() {
		return self::NAME;
	}

	/**
	 * @see \ManiaControl\Plugins\Plugin::getVersion()
	 */
	public static function getVersion() {
		return self::VERSION;
	}

	/**
	 * @see \ManiaControl\Plugins\Plugin::getAuthor()
	 */
	public static function getAuthor() {
		return self::AUTHOR;
	}

	/**
	 * @see \ManiaControl\Plugins\Plugin::getDescription()
	 */
	public static function getDescription() {
		return 'allows you to add guest who will not earn points in cup / round mode';
	}	
	
	/**
	 * Handle PlayerConnect callback
	 *
	 * @param Player $player
	 */
	public function handlePlayerConnect(Player $player) {
        if ($player->isSpectator) {
            $this->playerstate[$player->login] = -1;
        } else {
            $this->playerstate[$player->login] = 0;
            $this->nbplayers++;
        }
	}
	
	public function handlePlayerDisconnect(Player $player)
    {
        $this->playerstate[$player->login] = -1;
        $this->nbplayers--;
    }
	
	public function handlePlayerInfoChanged(Player $player)
    {

        $this->handlePlayerConnect($player);

        if ($this->playerstate[$player->login]) {
            $newSpecStatus = $player->isSpectator;

            if ($newSpecStatus) {
                if ($this->playerstate[$player->login] == 0 OR $this->playerstate[$player->login] == 1) {
                    $this->nbplayers--;
                }

                $this->playerstate[$player->login] = -1;
            } else {
                if ($this->playerstate[$player->login] == -1) {
                    $this->nbplayers++;
                }

                $this->playerstate[$player->login] = 0;
            }
        }

    }
	
	public function getPlayers()
    {
        return $this->playerstate;
    }
	
	/**
	 * Update Widgets on Setting Changes
	 *
	 * @param Setting $setting
	 */
	public function updateSettings(Setting $setting) {
	
	}
	
    /**
     * @param OnScoresStructure $structure
     */
    public function handleEndRoundCallback(OnScoresStructure $structure)
    {   $results = $structure->getPlayerScores();
		
		
		$alternpoint = false;
		$normalpoint = true;
		$playerpoint = 0;
		$guestpoint = 0;
		
		$nbguestdo = 0;
		
		
		$guest1 = $this->getGuest1();
		$guest2 = $this->getGuest2();
		$guest3 = $this->getGuest3();
		$guest4 = $this->getGuest4();
		$guest5 = $this->getGuest5();
		
		$guestsetting = $this->getGuestsetting();
		
		foreach ($results as $result) {
			$login = $result->getPlayer()->login;
			$rank = $result->getRank();
			$player = $result->getPlayer();
			
			
			if (($player->isSpectator && $result->getMatchPoints() == 0) || ($player->isFakePlayer() && $result->getMatchPoints() == 0)) {
			}elseif (($login == $guest1) || ($login == $guest2) || ($login == $guest3) || ($login == $guest4) || ($login == $guest5)) {
				if ($nbguestdo === 0){
					$oldpoints = $result->getRoundPoints();
					if ($guestsetting == false){
					
					}else{
						$this->maniaControl->getModeScriptEventManager()->setTrackmaniaPlayerPoints($player, $guestpoint);
				
					}
					$normalpoint = false;
					$nbguestdo = $nbguestdo + 1;
					
				}elseif ($nbguestdo === 1){ //2
					$oldpoints2 = $result->getRoundPoints();
					if ($guestsetting == false){
					
					}else{
						$this->maniaControl->getModeScriptEventManager()->setTrackmaniaPlayerPoints($player, $guestpoint);
				
					}
					$normalpoint = false;
					$nbguestdo = $nbguestdo + 1;
				}elseif ($nbguestdo === 2){ //3
					$oldpoints3 = $result->getRoundPoints();
					if ($guestsetting == false){
					
					}else{
						$this->maniaControl->getModeScriptEventManager()->setTrackmaniaPlayerPoints($player, $guestpoint);
				
					}
					$normalpoint = false;
					$nbguestdo = $nbguestdo + 1;
					
				}elseif ($nbguestdo === 3){ //4
					$oldpoints4 = $result->getRoundPoints();
					if ($guestsetting == false){
					
					}else{
						$this->maniaControl->getModeScriptEventManager()->setTrackmaniaPlayerPoints($player, $guestpoint);
				
					}
					$normalpoint = false;
					$nbguestdo = $nbguestdo + 1;
					
				}else{ //5+
					$oldpoints5 = $result->getRoundPoints();
					if ($guestsetting == false){
					
					}else{
						$this->maniaControl->getModeScriptEventManager()->setTrackmaniaPlayerPoints($player, $guestpoint);
				
					}
					$normalpoint = false;
					$nbguestdo = $nbguestdo + 1;
				}
				
			}elseif ($result->getRoundPoints() == 0){
				$roundpoints = $result->getRoundPoints();
				$points = $result->getMatchPoints();
				
			}else {
				if ($normalpoint === true) {
					$roundpoints = $result->getRoundPoints();
					$points = $result->getMatchPoints();
				}else {
					if ($nbguestdo = 1){
						$roundpoints = $result->getRoundPoints();
						$points = $result->getMatchPoints();
					
						$playerpoint = (($oldpoints - $roundpoints) + $roundpoints) + $points;
						$this->maniaControl->getModeScriptEventManager()->setTrackmaniaPlayerPoints($player, $playerpoint);
					
						$oldpoints = $result->getMatchPoints();
					
					}elseif ($nbguestdo = 2){
						$roundpoints = $result->getRoundPoints();
						$points = $result->getMatchPoints();
					
						$playerpoint = (($oldpoints - $roundpoints) + $roundpoints) + $points;
						$this->maniaControl->getModeScriptEventManager()->setTrackmaniaPlayerPoints($player, $playerpoint);
						
						$oldpoints = $oldpoints2;
						$oldpoints2 = $result->getMatchPoints();
						
					}elseif ($nbguestdo = 3){
						$roundpoints = $result->getRoundPoints();
						$points = $result->getMatchPoints();
					
						$playerpoint = (($oldpoints - $roundpoints) + $roundpoints) + $points;
						$this->maniaControl->getModeScriptEventManager()->setTrackmaniaPlayerPoints($player, $playerpoint);
						
						$oldpoints = $oldpoints2;
						$oldpoints2 = $oldpoints3;
						$oldpoints3 = $result->getMatchPoints();
						
					}elseif ($nbguestdo = 4){
						$roundpoints = $result->getRoundPoints();
						$points = $result->getMatchPoints();
					
						$playerpoint = (($oldpoints - $roundpoints) + $roundpoints) + $points;
						$this->maniaControl->getModeScriptEventManager()->setTrackmaniaPlayerPoints($player, $playerpoint);
						
						$oldpoints = $oldpoints2;
						$oldpoints2 = $oldpoints3;
						$oldpoints3 = $oldpoints4;
						$oldpoints4 = $result->getMatchPoints();
					}else{
						$roundpoints = $result->getRoundPoints();
						$points = $result->getMatchPoints();
					
						$playerpoint = (($oldpoints - $roundpoints) + $roundpoints) + $points;
						$this->maniaControl->getModeScriptEventManager()->setTrackmaniaPlayerPoints($player, $playerpoint);
						
						$oldpoints = $oldpoints2;
						$oldpoints2 = $oldpoints3;
						$oldpoints3 = $oldpoints4;
						$oldpoints4 = $oldpoints5;
						$oldpoints5 = $result->getMatchPoints();
					}
				}
			}
		}
		
		
		return true; 
		
	}	
	
	public function getGuest1()
    {
        return $this->maniaControl->getSettingManager()->getSettingValue(
            $this,
            self::SETTING_GUEST_PLUGIN_GUEST1
        );
    }
	
	public function getGuest2()
    {
        return $this->maniaControl->getSettingManager()->getSettingValue(
            $this,
            self::SETTING_GUEST_PLUGIN_GUEST2
        );
    }
	
	public function getGuest3()
    {
        return $this->maniaControl->getSettingManager()->getSettingValue(
            $this,
            self::SETTING_GUEST_PLUGIN_GUEST3
        );
    }
	
	public function getGuest4()
    {
        return $this->maniaControl->getSettingManager()->getSettingValue(
            $this,
            self::SETTING_GUEST_PLUGIN_GUEST4
        );
    }
	
	public function getGuest5()
    {
        return $this->maniaControl->getSettingManager()->getSettingValue(
            $this,
            self::SETTING_GUEST_PLUGIN_GUEST5
        );
    }
	
	public function getGuestsetting()
    {
        return $this->maniaControl->getSettingManager()->getSettingValue(
            $this,
            self::SETTING_GUEST_PLUGIN_GUESTPOINT
        );
    }
}
